--[[
	MiniMount
	Version 1.0
]]

MiniMount_Version = "1.0";

MiniMount_Mounts_Slow = {};
MiniMount_Mounts_Regular = {};
MiniMount_Mounts_Epic = {};
MiniMount_Mounts_FlyRegular = {};
MiniMount_Mounts_FlyEpic = {};
MiniMount_Mounts_Unknown = {};

local MiniMount_RidingSkill = 1;

-- Message zones
function MiniMount_Msg(msg, channel, debug)
	if (msg == nil) then
		return;
	end
	if debug == nil then
		debug = 10;
	end
	if channel ~= nil then
		channel = strlower(channel);
	end

	if (channel == "error") then
		DEFAULT_CHAT_FRAME:AddMessage(RED_FONT_COLOR_CODE..MINIMOUNT_TITLE.." "..MINIMOUNT_ERROR..": |r"..msg);
	elseif (channel == "debug") then
		DEFAULT_CHAT_FRAME:AddMessage(LIGHTYELLOW_FONT_COLOR_CODE..MINIMOUNT_TITLE.." "..MINIMOUNT_DEBUG..": |r"..msg);
	else
		DEFAULT_CHAT_FRAME:AddMessage(GREEN_FONT_COLOR_CODE..MINIMOUNT_TITLE..": |r"..msg);
	end
end

-- Loading Function
function MiniMount_Load()

	-- Slash Commands
	SlashCmdList["MiniMount"] = MiniMount_CommandLine;
	SLASH_MiniMount1 = "/minimount";
	SLASH_MiniMount2 = "/mount";
	SLASH_MiniMount3 = "/mm";

	if (not MiniMount_Options) then
		MiniMount_Options = {
			["DefaultMode"] = "Auto", -- Other values: "Regular", "Epic", "FlyRegular", FlyEpic"
		}
	end

	MiniMount_Msg(MINIMOUNT_VERSION.." "..MiniMount_Version.." "..MINIMOUNT_NOWLOADED);
	MiniMount_GetRidingSkill();
	MiniMount_BuildMountArrays();
end

function MiniMount_CommandLine(cmd)
	-- Force lowercase so we don't have to double-up our options! Also makes it more user-proof.
	cmd = strlower(cmd);

	if (cmd == "minimap off") then
		MiniMountMinimapButton:Hide();
		MiniMount_Msg("Hiding minimap button.");
	elseif (cmd == "minimap on") then
		MiniMountMinimapButton:Show();
		MiniMount_Msg("Showing minimap button.");
	elseif (cmd == "minimap") then
		if (MiniMountMinimapButton:IsVisible()) then
			MiniMountMinimapButton:Hide();
			-- MiniMount_Msg("Hiding minimap button.");
		else
			MiniMountMinimapButton:Show();
			-- MiniMount_Msg("Showing minimap button.");
		end
	elseif (cmd == "reg") or (cmd == "regular")  then
		MiniMount_Mount("Regular");
	elseif (cmd == "epic") then
		MiniMount_Mount("Epic");
	elseif (cmd == "fly reg") or (cmd == "fly regular") then
		MiniMount_Mount("FlyRegular");
	elseif (cmd == "fly epic") then
		MiniMount_Mount("FlyEpic");
	elseif (cmd == "fly") then
		MiniMount_Mount("Fly");
	elseif (cmd == "random") then
		MiniMount_Mount("Random");
	elseif (cmd == nil) or (cmd == "") then
		MiniMount_Mount();
	else
		MiniMount_Msg(MINIMOUNT_ERROR_WRONGOPTION,"error")
	end
end

function MiniMount_Mount(mode)
	MiniMount_BuildMountArrays();
	local MountNum = GetNumCompanions("MOUNT");
	local MountList = {};
	SetMapToCurrentZone();
	local continent = GetCurrentMapContinent();
	MiniMount_GetRidingSkill();

	if MountNum == 0 then
		MiniMount_Msg(MINIMOUNT_ERROR_NOMOUNTS,"error");
		return;
	end

	if MiniMount_RidingSkill < 75 then
		MiniMount_Msg(MINIMOUNT_ERROR_SKILLNOTHIGHENOUGH,"error");
		return;
	end

	if (not mode) then
		mode = MiniMount_Options["DefaultMode"];
	end

	if (not mode) then
		mode = "Auto";
	end

	local rand = {
		[1] = "Auto",
	}
	if #MiniMount_Mounts_Slow > 0 then
		tinsert(rand, (#rand + 1), "Slow");
	end
	if #MiniMount_Mounts_Regular > 0 then
		tinsert(rand, (#rand + 1), "Regular");
	end
	if #MiniMount_Mounts_Epic > 0 then
		tinsert(rand, (#rand + 1), "Epic");
	end
	if (continent == 3) or ((continent == 4) and (UnitLevel("player") >= 77)) then
		if #MiniMount_Mounts_FlyRegular > 0 then
			tinsert(rand, (#rand + 1), "FlyRegular");
		end
		if #MiniMount_Mounts_FlyEpic > 0 then
			tinsert(rand, (#rand + 1), "FlyEpic");
		end
	end
	if #MiniMount_Mounts_Unknown > 0 then
		tinsert(rand, (#rand + 1), "Unknown");
	end

	if mode == "Random" then
		mode = rand[random(1,#rand)];
	end

	if mode == "Auto" then
		if MiniMount_RidingSkill == 75 then
			mode = "Regular";
		elseif MiniMount_RidingSkill >= 150 then
			mode = "Epic";
		end

		if (continent == 3) or ((continent == 4) and (UnitLevel("player") >= 77)) then
			if (MiniMount_RidingSkill == 225) then
				mode = "FlyRegular";
			elseif (MiniMount_RidingSkill == 300) then
				mode = "FlyEpic";
			end
		end
	end

	if (mode == "Fly") then
		if (MiniMount_RidingSkill == 225) then
			mode = "FlyRegular";
		elseif (MiniMount_RidingSkill == 300) then
			mode = "FlyEpic";
		else
			if MiniMount_RidingSkill == 75 then
				mode = "Regular";
			elseif MiniMount_RidingSkill >= 150 then
				mode = "Epic";
			end
		end
	end

	MountList = getglobal("MiniMount_Mounts_"..mode);
	local chk_flye, chk_flys, chk_ge, chk_gs = nil, nil, nil, nil;
	while (type(MountList) == nil) do
		if (continent == 3) or ((continent == 4) and (UnitLevel("player") >= 77)) then
			if chk_flye == nil then
				if (#MiniMount_Mounts_FlyEpic > 0) then
					MountList = MiniMount_Mounts_FlyEpic;
				else
					chk_flye = 1;
				end
			elseif chk_flys == nil then
				if (#MiniMount_Mounts_FlyRegular > 0) then
					MountList = MiniMount_Mounts_FlyRegular;
				else
					chk_flys = 1;
				end
			elseif chk_ge == nil then
				if (#MiniMount_Mounts_Epic > 0) then
					MountList = MiniMount_Mounts_Epic;
				else
					chk_ge = 1;
				end
			else
				if (#MiniMount_Mounts_Regular > 0) then
					MountList = MiniMount_Mounts_Regular;
				else
					chk_gs = 1;
				end
			end
		else
			if chk_ge == nil then
				if (#MiniMount_Mounts_Epic > 0) then
					MountList = MiniMount_Mounts_Epic;
				else
					chk_ge = 1;
				end
			elseif chk_gs == nil then
				if (#MiniMount_Mounts_Regular > 0) then
					MountList = MiniMount_Mounts_Regular;
				else
					chk_ge = 1;
				end
			end
		end
		if chk_flye == 1 and chk_flys == 1 and chk_ge == 1 and chk_gs == 1 then
			MiniMount_Msg(MINIMOUNT_ERROR_MOUNTDATABASEEMPTY);
			return;
		end
	end

	local mymount = MountList[random(1,#MountList)];

	if not IsFlying() then
		-- local _, MountName = GetCompanionInfo("MOUNT",mymount);
		-- MiniMount_Msg("Summoning "..MountName.."...");
		CallCompanion("MOUNT", mymount);
	else
		MiniMount_Msg(MINIMOUNT_ERROR_ISFLYING);
	end
end

function MiniMount_BuildMountArrays()
	local num = GetNumCompanions("MOUNT");

	if num == 0 then
		return;
	end

	local mymounts = {};

	for x=1, num do
		local _, MountName, MountSID = GetCompanionInfo("MOUNT",x);
		
		for y=1,#MiniMount_Mounts["Slow"] do
			if MiniMount_Mounts["Slow"][y] == tostring(MountSID) then
				tinsert(MiniMount_Mounts_Slow, (#MiniMount_Mounts_Slow+1), x);
			end
		end
		for y=1,#MiniMount_Mounts["Regular"] do
			if MiniMount_Mounts["Regular"][y] == tostring(MountSID) then
				tinsert(MiniMount_Mounts_Regular, (#MiniMount_Mounts_Regular+1), x);
			end
		end
		for y=1,#MiniMount_Mounts["Epic"] do
			if MiniMount_Mounts["Epic"][y] == tostring(MountSID) then
				tinsert(MiniMount_Mounts_Epic, (#MiniMount_Mounts_Epic+1), x);
			end
		end
		for y=1,#MiniMount_Mounts["FlyRegular"] do
			if MiniMount_Mounts["FlyRegular"][y] == tostring(MountSID) then
				tinsert(MiniMount_Mounts_FlyRegular, (#MiniMount_Mounts_FlyRegular+1), x);
			end
		end
		for y=1,#MiniMount_Mounts["FlyEpic"] do
			if MiniMount_Mounts["FlyEpic"][y] == tostring(MountSID) then
				tinsert(MiniMount_Mounts_FlyEpic, (#MiniMount_Mounts_FlyEpic+1), x);
			end
		end
		for y=1,#MiniMount_Mounts["Shifting"] do
			if MiniMount_Mounts["Epic"][y] == tostring(MountSID) then
				if MiniMount_RidingSkill < 150 then
					tinsert(MiniMount_Mounts_Regular, (#MiniMount_Mounts_Regular+1), x);
				else
					tinsert(MiniMount_Mounts_Epic, (#MiniMount_Mounts_Epic+1), x);
				end
			end
		end
		for y=1,#MiniMount_Mounts["FlyShifting"] do
			if MiniMount_Mounts["Epic"][y] == tostring(MountSID) then
				if MiniMount_RidingSkill < 150 then
					tinsert(MiniMount_Mounts_Regular, (#MiniMount_Mounts_Regular+1), x);
				else
					tinsert(MiniMount_Mounts_Epic, (#MiniMount_Mounts_Epic+1), x);
				end
				if MiniMount_RidingSkill < 300 then
					tinsert(MiniMount_Mounts_FlyRegular, (#MiniMount_Mounts_FlyRegular+1), x);
				else
					tinsert(MiniMount_Mounts_FlyEpic, (#MiniMount_Mounts_FlyEpic+1), x);
				end
			end
		end

		local count1, count2, count3, count4, count5 = 0,0,0,0,0;

		for z=1,#MiniMount_Mounts_Slow do
			if MiniMount_Mounts_Slow[z] == x then
				count1 = count1 + 1;
			end
		end
		for z=1,#MiniMount_Mounts_Regular do
			if MiniMount_Mounts_Regular[z] == x then
				count2 = count2 + 1;
			end
		end
		for z=1,#MiniMount_Mounts_Epic do
			if MiniMount_Mounts_Epic[z] == x then
				count3 = count3 + 1;
			end
		end
		for z=1,#MiniMount_Mounts_FlyRegular do
			if MiniMount_Mounts_FlyRegular[z] == x then
				count4 = count4 + 1;
			end
		end
		for z=1,#MiniMount_Mounts_FlyEpic do
			if MiniMount_Mounts_FlyEpic[z] == x then
				count5 = count5 + 1;
			end
		end

		if (count1==0) and (count2==0) and (count3==0) and (count4==0) and (count5==0) then
			tinsert(MiniMount_Mounts_Unknown, x);
			MiniMount_Msg(MINIMOUNT_UNKNOWNMOUNTFOUND.."SpellID "..MountSID.." = "..MountName);
		end
	end
end

function MiniMount_GetRidingSkill()
	local num = GetNumSkillLines();

	for x=1,num do
		local skillName, isHeader, _, skillRank = GetSkillLineInfo(x);
		if isHeader == nil then
			if skillName == "Riding" then
				MiniMount_RidingSkill = skillRank;
			end
		end
	end
end

-- Let's us sort tables!
-- Copy/Pasted from LUA Website
function MiniMount_pairsByKeys (t, f)
	local a = {}
		for n in pairs(t) do table.insert(a, n) end
		table.sort(a, f)
		local i = 0      -- iterator variable
		local iter = function ()   -- iterator function
			i = i + 1
			if a[i] == nil then return nil
			else return a[i], t[a[i]]
			end
		end
	return iter
end